const { Command } = require('commander');
const packageJSON = require('./package.json');
const chalk = require('chalk');
const fs = require('fs-extra');
const spawn = require('cross-spawn');
const path = require('path');
const inquirer = require('inquirer');
const ora = require('ora')

const program = new Command();
const templateName = {
  h5_webpack: '@tun/h5-template',
  h5_vite: '@tun/h5-template-vite',
  pc_webpack: '@tun/h5-template',
  pc_vite: '@tun/h5-template-vite',
  node: '@tun/node-template',
}
async function init() {
  program
    .name(packageJSON.name)
    .usage('<command> [option]')
    .description('CLI to Tuntun')
    .version(packageJSON.version);
  
  program.command('create')
    .description('Create a Tuntun project.')
    .argument('<project-directory>', 'project directory')
    // .option("-H, --h5", "H5 template")
    // .option("-P, --pc", "pc template")
    // .option("-N, --node", "Node template")
    .action(( appName) => {
      // console.log('projectName', appName);
      createProject(appName)
    })

  program.parse(process.argv);
}

async function createProject(appName) {
  validateProjectName(appName);
  const targetPath = path.resolve(appName)
  if (fs.existsSync(targetPath)) {
    const answer = await inquirer.prompt([
      {
        type: 'confirm',
        message: 'The current file already exists. Do you want to overwrite it?',
        default: false,
        name: 'overwrite'
      }
    ]);
    // console.log(answer)
    if (answer.overwrite) {
      fs.remove(targetPath)
    } else {
      return
    }
  }
  fs.ensureDirSync(appName);
  let key = ''
  const answer = await inquirer.prompt([
    {
      type: 'list',
      message: 'Which type of template to choose?',
      name: 'type',
      choices: [
        {
          name: 'H5',
          value: 'h5'
        },
        {
          name: 'PC',
          value: 'pc'
        },
        {
          name: 'Node',
          value: 'node'
        }
      ]
    }
  ]);
  key = answer.type
  if (answer.type === 'h5' || answer.type === 'pc') {
    const answer = await inquirer.prompt([
      {
        type: 'list',
        message: 'Which build tool to choose?',
        name: 'tool',
        choices: [
          {
            name: 'Webpack',
            value: 'webpack'
          },
          {
            name: 'Vite',
            value: 'vite'
          }
        ]
      }
    ]);
    key = `${key}_${answer.tool}`
  }
  // console.log("key", key)
  const spinner = ora(`Creating a new project at ${chalk.green(targetPath)}.`).start();
  // console.log('\n targetPath', targetPath)
  try {
    spinner.text = 'Creating package.json...'
    const packageJSON = {
      name: appName,
      version: '0.1.0',
      private: true
    }
    fs.writeFileSync(
      path.join(targetPath, 'package.json'),
      JSON.stringify(packageJSON, null, 2)
    );
    // change directory 
    process.chdir(targetPath);
    // console.log("root", targetPath)
    spinner.text = 'Installing...'
    const packageName = templateName[key]
    await install(targetPath, packageName);
    copyTemplate(targetPath, packageName, appName)
    spinner.succeed(`🎉 Success! Created ${chalk.green(appName)} at ${targetPath}`);
    console.log('Done, now run:')
    console.log(chalk.green(`\n cd ${appName}`))
    console.log(chalk.green(` npm install`))
    console.log(chalk.green(` npm run start:dev\n`))
    process.exit(0);
  } catch(err) {
    // spinner.fail(err)
    spinner.fail('Create failed. Please try again later.')
    process.exit(0);
  }
}

/**
 * @description - validate ProjectName
 * 1. CamelCase
 * 2. Cannot start with . or _
 * 3. Cannot contain ~)('!*
 * 4. Max length: 214
 */
function validateProjectName(appName) {
  // console.log(/^[a-z\d]+([A-Z][a-z\d]{1,})*$/g.test(appName))
  if (String(appName).length > 214 || !/^[a-z\d]+([A-Z][a-z\d]{1,})*$/g.test(appName)) {
    console.log(
      chalk.red(
        `Cannot create a project named ${chalk.green(
          `"${appName}"`
        )} because of project naming restrictions:`
      )
    );
    console.log(
      chalk.red('\n * CamelCase')
    );
    console.log(
      chalk.red('\n * Cannot start with . or _')
    );
    console.log(
      chalk.red("\n * Cannot contain ~)('!*")
    );
    console.log(
      chalk.red('\n * Max length: 214')
    );
    console.log(chalk.red('\nPlease choose a different project name.'));
    process.exit(1);
  }
}

async function install(targetPath, packageName) {
  return new Promise((resolve, reject) => {
    const child = spawn('npm', ['install', packageName], {
      cwd: targetPath,
      shell: true,
      stdio: 'inherit'
    });
    child.on('close', (code) => {
      if (code === 0) {
        resolve()
      } else {
        console.log(chalk.red(`${packageName} installation failed. Please check the network and try again.`));
        reject()
      }
    })
  })
}

async function copyTemplate(targetPath, packageName, appName) {
  // remove package.json、package-lock.json
  fs.removeSync(path.join(targetPath, 'package.json'));
  fs.removeSync(path.join(targetPath, 'package-lock.json'));
  // copy template to targetPath
  fs.copySync(path.join(targetPath, `node_modules/${packageName}/template`), targetPath);
  // remove node_modules
  fs.removeSync(path.join(targetPath, 'node_modules'));
  // Read package.json
  const packageJson = JSON.parse(fs.readFileSync(path.join(targetPath, 'package.json')));
  // Replace name field in package.json
  packageJson.name = appName;
  
  // Write modified package.json
  fs.writeFileSync(packageJsonPath, JSON.stringify(packageJson, null, 2));
  // remove package-lock.json
  fs.removeSync(path.join(targetPath, 'package-lock.json'));
}
module.exports = {
  init
}